package com.hys.app.service.erp;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hys.app.framework.database.BaseQueryParam;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseService;
import com.hys.app.model.erp.dos.GoodsDO;
import com.hys.app.model.erp.dto.GoodsDTO;
import com.hys.app.model.erp.dto.GoodsQueryParams;
import com.hys.app.model.erp.dto.GoodsSortDto;
import com.hys.app.model.erp.vo.GoodsVO;
import com.hys.app.model.erp.vo.ProductStockVO;
import com.hys.app.model.erp.vo.ProductVO;

import java.util.List;

/**
 * 商品业务层接口
 *
 * @author 张崧
 * 2023-12-26 15:56:58
 */
public interface GoodsManager extends BaseService<GoodsDO> {
    /**
     * 分页列表
     *
     * @param queryParams 查询条件
     * @return 分页数据
     */
    WebPage<GoodsVO> list(GoodsQueryParams queryParams);


    /**
     * 添加
     *
     * @param goodsDTO
     */
    void add(GoodsDTO goodsDTO);


    GoodsVO getInfo(Long id);

    /**
     * 编辑
     *
     * @param goodsDTO
     */
    void edit(GoodsDTO goodsDTO);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    GoodsVO getDetail(Long id);

    /**
     * 删除
     *
     * @param id
     */
    void delete(Long id);

    /**
     * 根据商品分类查询数量
     *
     * @param categoryId
     * @return
     */
    long countByCategoryId(Long categoryId);

    /**
     * 上下架前判断该商品在各个仓库还有没有库存
     *
     * @param id 商品id
     * @return 有库存返回true   无返回false
     */
    Boolean updateCheck(Long id);

    /**
     * 根据仓库获取仓库所有sku的库存
     *
     * @param warehouseId
     * @return
     */
    Page<ProductStockVO> getSkuByWarehouse(Long wareHouseId, BaseQueryParam queryParam);

    void updateSort(GoodsSortDto request);

}

