package com.hys.app.service.erp;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseService;
import com.hys.app.model.erp.dos.GoodsDO;
import com.hys.app.model.erp.dos.ProductDO;
import com.hys.app.model.erp.dos.ProductGroupDO;
import com.hys.app.model.erp.dto.*;
import com.hys.app.model.erp.vo.ProductStockVO;
import com.hys.app.model.erp.vo.ProductVO;
import com.hys.app.model.erp.vo.TransitVO;

import java.util.List;

/**
 * 产品业务层接口
 *
 * @author 张崧
 * 2023-11-30 16:06:44
 */
public interface ProductManager extends BaseService<ProductDO> {
    /**
     * 分页列表
     *
     * @param queryParams 查询条件
     * @return 分页数据
     */
    WebPage<ProductVO> list(ProductQueryParams queryParams);

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    ProductVO getDetail(String id);

    /**
     * 添加
     */
    void add(Long id);

    /**
     * 编辑
     *
     * @param goodsDO
     * @param productList
     */
    void edit(GoodsDO goodsDO, List<ProductDTO> productList);

    /**
     * 根据商品id查询
     *
     * @param goodsId
     * @return
     */
    List<ProductDO> listByGoodsId(Long goodsId);

    /**
     * 根据商品id删除
     *
     * @param goodsIds
     */
    void deleteByGoodsId(List<Long> goodsIds);

    /**
     * 产品分组
     *
     * @param productGroupDO
     */
    void productGroup(ProductGroupDO productGroupDO);

    /**
     * 删除产品分组
     *
     * @param id
     */
    void removeProductGroup(Long id);

    /**
     * 修改产品分组
     *
     * @param id
     * @param productGroupDO
     */
    void updateProductGroup(Long id, ProductGroupDO productGroupDO);

    /**
     * 获取产品分组列表
     *
     * @param queryParams
     * @return
     */
    Page<ProductGroupDO> getGroupPage(ProductGroupQueryParams queryParams);

    /**
     * 获取产品分组详情
     *
     * @param id
     * @return
     */
    ProductGroupDO getGroupById(Long id);

    /**
     * 根据分区获取分组所有的sku信息
     *
     * @param groupId
     * @return
     */
    List<ProductStockVO> getProductByGroup(Long groupId, List<Long> warehouseId);

    /**
     * 设置库存预警
     *
     * @param stockId
     * @param warnNum
     * @return
     */
    void updateWarnStock(List<Long> stockId, Integer warnNum);

    /**
     * 首页获取库存预警列表
     *
     * @param warehouseId
     */
    List<ProductStockVO> getWarnStock(Long warehouseId);

    /**
     * 获取在途明细 去发货列表里面找
     *
     * @param stockId
     * @return
     */
    List<TransitVO> transitDetail(Long stockId);

    /**
     * 全局更新sku排序
     *
     * @param sortItem
     * @return
     */
    void updateSort(List<SortItem>  sortItem);
}

