package com.hys.app.service.erp;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hys.app.framework.database.BaseQueryParam;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseService;
import com.hys.app.model.erp.dos.GoodsDO;
import com.hys.app.model.erp.dos.WarehouseAreaDO;
import com.hys.app.model.erp.dos.WarehouseOutItemDO;
import com.hys.app.model.erp.dto.WarehouseAreaQueryParams;
import com.hys.app.model.erp.dto.WarehouseOutItemQueryParams;
import com.hys.app.model.erp.vo.ProductStockVO;
import com.hys.app.model.erp.vo.WarehouseAreaVO;
import com.hys.app.model.erp.vo.WarehouseOutItemVO;

import java.util.List;

/**
 * 出库单商品明细业务层接口
 *
 * @author 张崧
 * @since 2023-12-07 17:06:32
 */
public interface WarehouseAreaManager extends BaseService<WarehouseAreaDO> {
    /**
     * 分页列表
     *
     * @param queryParams 查询条件
     * @return 分页数据
     */
    WebPage<WarehouseAreaVO> list(WarehouseAreaQueryParams queryParams);

    WarehouseAreaVO getDetail(Long id);

    void delete(Long id);

    void add(WarehouseAreaDO warehouseAreaDO);

    void edit(WarehouseAreaDO warehouseAreaDO);

    /**
     * 获取所有没有绑定枯萎的商品
     *
     * @param baseQueryParam
     * @return
     */
    Page<ProductStockVO> goodNotAreaList(BaseQueryParam baseQueryParam, Long warehouseId);

    /**
     * 根据仓库id获取仓库下所有的库位
     *
     * @param warehouseId
     * @return
     */
    List<WarehouseAreaDO> getAreaByWarehouse(Long warehouseId);

    /**
     * 库存绑定库位
     * 保存以最后一次带过来的数据为主，之前的全部删掉
     *
     * @param warehouseId
     * @param warehouseAreaId
     * @param stockIdList
     */
    void bindAreaStock(Long warehouseId, Long warehouseAreaId, List<Long> stockIdList);
}

